#!/usr/bin/env python

import pygame
import random
import os.path
from math import sqrt

RIGHT = 0
LEFT = 1

SIZE_W = 10
SIZE_H = 9
TILE = 16
SCALE = 5
SCALED_TILE = TILE * SCALE

COLOR0 = (8, 24, 32)
COLOR1 = (48, 104, 80)
COLOR2 = (136, 192, 112)
COLOR3 = (232, 255, 212)
COLOR_ALPHA = (0, 128, 128)

class Space(pygame.sprite.Sprite):
	def __init__(self):
		pygame.sprite.Sprite.__init__(self)
		self.image = pygame.transform.scale(pygame.image.load(os.path.join('data', 'space.bmp')), (1024 * SCALE, 1024 * SCALE))
		self.rect = pygame.Rect((0, 0), self.image.get_size())
		self.speed_surface = pygame.Surface((SIZE_W * TILE, SIZE_H * TILE))
		self.speed_surface.set_colorkey((0, 128, 128))
		offset = SCALED_TILE * 2
		self.scroll_rect = pygame.Rect((offset, offset), (SIZE_W * SCALED_TILE - offset * 2, SIZE_H * SCALED_TILE - offset * 2))
		self.x_vel = 0
		self.y_vel = 0
	
	def load(self, objects):
		objects['planet'].add(Planet((random.randint(-200000, 200000), random.randint(-200000, 200000))))
		return
	
	def update(self, screen, objects):
		self.rect.x += self.x_vel
		self.rect.y += self.y_vel

		self.rect.x %= 1024 * SCALE
		self.rect.y %= 1024 * SCALE

		'''
		if self.rect.x < 0:
			x_rect = pygame.Rect(self.rect)
			x_rect.x += 1024 * SCALE + 1
			screen.blit(self.image, (0, 0), x_rect)
		'''
		if self.rect.x > 1024 * SCALE - SIZE_W * SCALED_TILE:
			x_rect = pygame.Rect(self.rect)
			x_rect.x -= 1024 * SCALE
			screen.blit(self.image, (0, 0), x_rect)

		if self.rect.y > 1024 * SCALE - SIZE_H * SCALED_TILE:
			y_rect = pygame.Rect(self.rect)
			y_rect.y -= 1024 * SCALE
			screen.blit(self.image, (0, 0), y_rect)

		if self.rect.x > 1024 * SCALE - SIZE_W * SCALED_TILE and self.rect.y > 1024 * SCALE - SIZE_H * SCALED_TILE:
			xy_rect = pygame.Rect(self.rect)
			xy_rect.x -= 1024 * SCALE
			xy_rect.y -= 1024 * SCALE
			screen.blit(self.image, (0, 0), xy_rect)

		'''
		if self.rect.y > 0:
			screen.blit(self.image, (0, 0), self.rect)
		if self.rect.y > 1024 + SIZE_W * SCALED_TILE:
			self.rect.y -= 1024
		'''

		screen.blit(self.image, (0, 0), self.rect)

		speed = abs(self.x_vel) + abs(self.y_vel)
		if speed > 50:
			self.speed_surface.fill((0, 128, 128))

			for i in range(16):
				pos_x = random.randint(0, SIZE_W * TILE)
				pos_y = random.randint(0, SIZE_H * TILE)
				pygame.draw.line(self.speed_surface, COLOR3, (pos_x, pos_y), (pos_x - self.x_vel / 10, pos_y - self.y_vel / 10))
			scaled_speed_surface = pygame.transform.scale(self.speed_surface, (SIZE_W * SCALED_TILE, SIZE_H * SCALED_TILE))
			screen.blit(scaled_speed_surface, (0, 0))


class Planet(pygame.sprite.Sprite):
	def __init__(self, (x, y)):
		pygame.sprite.Sprite.__init__(self)
		self.image = pygame.transform.scale(pygame.image.load(os.path.join('data', 'planet.bmp')).convert(), (SIZE_W * SCALED_TILE, SIZE_H * SCALED_TILE))
		self.image.set_colorkey((0, 128, 128))
		self.rect = pygame.Rect((x, y), self.image.get_size())

	def update(self, screen, objects):
		space = objects['space']

		self.rect.x -= objects['space'].x_vel
		self.rect.y -= objects['space'].y_vel

		# Blit
		screen.blit(self.image, self.rect)


class Boost(pygame.sprite.Sprite):
	def __init__(self, (x, y), (x_vel, y_vel)):
		pygame.sprite.Sprite.__init__(self)
		self.image = pygame.transform.scale(pygame.image.load(os.path.join('data', 'boost.bmp')).convert(), (SCALED_TILE, SCALED_TILE / 4))
		self.image.set_colorkey((0, 128, 128))
		self.rect = pygame.Rect((x, y), (SCALED_TILE / 4, SCALED_TILE / 4))
		self.x_vel = x_vel
		self.y_vel = y_vel
		self.anim_timer = 0
		self.frame = 0
	
	def update(self, screen, objects):
		self.rect.x += self.x_vel #- objects['space'].x_vel
		self.rect.y += self.y_vel #- objects['space'].y_vel

		self.anim_timer += 1
		if self.anim_timer > 5:
			self.frame += 1
			self.anim_timer = 0
			if self.frame > 3:
				self.kill()

		rect = pygame.Rect((self.frame * SCALED_TILE / 4, 0), (SCALED_TILE / 4, SCALED_TILE / 4))
		screen.blit(self.image, self.rect.center, rect)
		

class Player(pygame.sprite.Sprite):
	def __init__(self, (x, y)):
		pygame.sprite.Sprite.__init__(self)
		self.image = pygame.transform.scale(pygame.image.load(os.path.join('data', 'player.bmp')).convert(), (SCALED_TILE * 4, SCALED_TILE * 4))
		self.image.set_colorkey((0, 128, 128))
		self.rect = pygame.Rect((x, y), (SCALED_TILE, SCALED_TILE))
		self.x_vel = 0
		self.y_vel = 0
		self.anim_timer = 0
		self.frame = 0
		self.latency = 0
		self.direction = RIGHT
	
	def update(self, screen, objects, game_over, win):
		space = objects['space']
		if self.latency <= 0 and not game_over and not win:
			pressed = pygame.key.get_pressed()
			px = pressed[pygame.K_RIGHT] - pressed[pygame.K_LEFT]
			py = pressed[pygame.K_DOWN] - pressed[pygame.K_UP]
			self.x_vel += px
			self.y_vel += py
			if px:
				self.direction = px < 0
			if px or py:
				d = -1
				if self.direction == LEFT:
					d = 1
				objects['effect'].add(Boost((self.rect.x + d * 24, self.rect.y), (-px * 2, -py * 2)))

			'''
			for planet in objects['planet'].sprites():
				planet_x = planet.rect.x
				planet_y = planet.rect.y
				x_vel = float(planet_x) / float(abs(planet_x)) * 0.2
				y_vel = float(planet_y) / float(abs(planet_y)) * 0.2
				self.x_vel += x_vel
				self.y_vel += y_vel
			'''

			self.latency += 10
		else:
			self.latency -= 1


		# Move
		self.rect.x += self.x_vel
		if not space.scroll_rect.contains(self.rect):
			self.rect.x -= self.x_vel
			space.x_vel += self.x_vel
			self.x_vel = 0
		'''
		else:
			space.x_vel = 0
		'''

		self.rect.y += self.y_vel
		if not space.scroll_rect.contains(self.rect):
			self.rect.y -= self.y_vel
			space.y_vel += self.y_vel
			self.y_vel = 0
		'''
		else:
			space.y_vel = 0
		'''

		# Anim
		self.anim_timer += 1
		if self.anim_timer > 5:
			self.frame += 1
			if self.frame > 3: self.frame = 0
			self.anim_timer = 0

		self.frame = 0

		# Blit
		rect = pygame.Rect(self.frame * SCALED_TILE, self.direction * SCALED_TILE, SCALED_TILE, SCALED_TILE)
		pos = (self.rect.x - 4 * SCALE, self.rect.y - 4 * SCALE)
		screen.blit(self.image, pos, rect)


def load_game(objects):
	objects['player'].empty()
	objects['planet'].empty()
	objects['effect'].empty()
	objects['player'].add(Player((SIZE_W * SCALED_TILE / 2, SIZE_H * SCALED_TILE / 2)))
	objects['space'].load(objects)
	objects['space'].x_vel = 0
	objects['space'].y_vel = 0


def main():
	pygame.init()
	size = (SIZE_W * SCALED_TILE, SIZE_H * SCALED_TILE)

	screen = pygame.display.set_mode(size, pygame.DOUBLEBUF)
	pygame.mouse.set_visible(False)

	font = pygame.font.Font(None, 32)

	space = Space()
	effect_group = pygame.sprite.Group()
	planet_group = pygame.sprite.Group()
	player_group = pygame.sprite.Group()
	objects = {'space': space, 'planet': planet_group, 'player': player_group, 'effect': effect_group}
	load_game(objects)

	game_info = {'score': 0, 'highscore': 0}

	run = True
	win = False
	game_over = False
	clock = pygame.time.Clock()
	timer = 0
	oxygen = 100

	shot = 0
	capture = False

	while run:
		clock.tick(60)
		timer += 1

		if not timer % 120 and oxygen > 0 and not win:
			oxygen -= 1
		elif oxygen == 0:
			game_over = True

		caption = "In Space [FPS:%.2f]" % clock.get_fps()
		pygame.display.set_caption(caption)

		for event in pygame.event.get():
			if event.type == pygame.QUIT: run = False
			if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE: run = False
			if event.type == pygame.KEYDOWN and event.key == pygame.K_c: capture = not(capture)
			if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN and game_over:
				load_game(objects)
				oxygen = 100
				game_over = False

		screen.fill(COLOR0)
		objects['space'].update(screen, objects)
		objects['planet'].update(screen, objects)
		objects['effect'].update(screen, objects)
		objects['player'].update(screen, objects, game_over, win)

		off_x = objects['planet'].sprites()[0].rect.x
		off_y = objects['planet'].sprites()[0].rect.y

		distance = sqrt(off_x * off_x + off_y * off_y)
		if distance < 200 and not game_over:
			objects['player'].sprites()[0].x_vel = 0
			objects['player'].sprites()[0].y_vel = 0
			objects['space'].x_vel = 0
			objects['space'].y_vel = 0
			win = True

		text = font.render("Objective at %i" % (distance), 0, COLOR3).convert()
		text_scaled = pygame.transform.scale(text, (text.get_width() * SCALE / 2, text.get_height() * SCALE / 2))
		screen.blit(text, (10, 10))

		text = font.render("O2: %i %%" % oxygen, 0, COLOR3).convert()
		text_scaled = pygame.transform.scale(text, (text.get_width() * SCALE / 2, text.get_height() * SCALE / 2))
		screen.blit(text, (10, 40))

		if win:
			text = font.render("You win!", 0, COLOR3).convert()
			text_scaled = pygame.transform.scale(text, (text.get_width() * SCALE / 2, text.get_height() * SCALE / 2))
			pos = (screen.get_width() / 2 - text_scaled.get_width() / 2, screen.get_height() / 2 - text_scaled.get_height() / 2)
			screen.blit(text_scaled, pos)
		elif game_over:
			text = font.render("You've run out of oxygen...", 0, COLOR3).convert()
			text_scaled = pygame.transform.scale(text, (text.get_width() * SCALE / 2, text.get_height() * SCALE / 2))
			pos = (screen.get_width() / 2 - text_scaled.get_width() / 2, screen.get_height() / 2 - text_scaled.get_height() / 2)
			screen.blit(text_scaled, pos)

			text = font.render("Press enter to try again.", 0, COLOR3).convert()
			text_scaled = pygame.transform.scale(text, (text.get_width() * SCALE / 4, text.get_height() * SCALE / 4))
			pos = (screen.get_width() / 2 - text_scaled.get_width() / 2, screen.get_height() / 2 - text_scaled.get_height() / 2 + 50)
			screen.blit(text_scaled, pos)

		if capture and 0:
			shot += 1
			pygame.image.save(screen, "gif/capture%i.png" % shot)

		pygame.display.flip()

	pygame.quit()

if __name__ == "__main__":
	main()
